package com.usky.dxtop.service.listener;

import com.alibaba.fastjson.JSONObject;
import com.usky.dxtop.model.Dish;
import com.usky.dxtop.service.DishService;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.core.util.JsonUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.Optional;

/**
 * @author yq
 * @date 2021/9/9 11:05
 */
@Slf4j
@Component
public class DishMqListener {

    @Autowired
    private DishService dishService;


    @RabbitListener(queues = "dish_trade_produce", containerFactory = "dishListenerFactory")
    public void dealDeclareMessage(Message message) throws Exception {
        try {
            //消息类型
            String routingKey = message.getMessageProperties().getReceivedRoutingKey();
            log.info(routingKey);
            //消息内容
            String s = new String(message.getBody(), StandardCharsets.UTF_8);
            log.info("dish_trade_produce获取到的消息"+s);
            Dish dish = checkDate(s);
            dishService.saveOrUpdate(dish);
        } catch (Exception e) {
            log.info("dish_trade_produce"+"异常信息：" + e.getMessage());
        } finally {

        }
    }


    public Dish checkDate(String message){
        JSONObject jsonObject = JSONObject.parseObject(message);
        Dish dish = JSONObject.parseObject(message, Dish.class);
        JSONObject details = jsonObject.getJSONObject("details");
        Optional.ofNullable(details)
                .ifPresent(de -> {
                    Optional.ofNullable(de.get("pid")).ifPresent(pId -> dish.setPId(Integer.parseInt(pId.toString())));
                    Optional.ofNullable(de.get("name")).ifPresent(pName -> dish.setPName(pName.toString()));
                    Optional.ofNullable(de.get("price")).ifPresent(pPrice -> dish.setPPrice(new BigDecimal(pPrice.toString())));
                    Optional.ofNullable(de.get("amt")).ifPresent(pAmt -> dish.setPAmt(new BigDecimal(pAmt.toString())));
                });
        JSONObject payments = jsonObject.getJSONObject("payments");
        Optional.ofNullable(payments)
                .ifPresent(de -> {
                    Optional.ofNullable(de.get("type")).ifPresent(type -> dish.setPaymentType(Integer.parseInt(type.toString())));
                    Optional.ofNullable(de.get("amt")).ifPresent(amt -> dish.setPaymentAmt(new BigDecimal(amt.toString())));
                    Optional.ofNullable(de.get("balance")).ifPresent(balance -> dish.setPaymentBalance(new BigDecimal(balance.toString())));
                });
        return dish;
    }
}
